""" Local resolver API
"""

from logging import getLogger

from core_linked_records_app import settings
from core_linked_records_app.components.data.access_control import (
    can_get_pid_for_data,
    can_get_data_by_pid,
)
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_linked_records_app.utils.dict import (
    is_dot_notation_in_dictionary,
    get_value_from_dot_notation,
)
from core_linked_records_app.utils.pid import is_valid_pid_value
from core_main_app.access_control.decorators import access_control
from core_main_app.commons.exceptions import ApiError, DoesNotExist
from core_main_app.components.data import api as data_api

logger = getLogger(__name__)


@access_control(can_get_data_by_pid)
def get_data_by_pid(pid, request):
    """Return data object with the given pid.

    Parameters:
        pid:
        request: HttpRequest

    Returns: data object
    """
    try:
        pid_path_list = [
            pid_path_object.path
            for pid_path_object in pid_path_api.get_all(request)
        ] + [settings.PID_PATH]
        dot_notation_pid_paths = [
            f"dict_content.{pid_path}" for pid_path in pid_path_list
        ]
        query_result = data_api.execute_json_query(
            {
                "$or": [
                    {dot_notation_pid_path: pid}
                    for dot_notation_pid_path in dot_notation_pid_paths
                ]
            },
            request.user,
        )
        query_result_length = len(query_result)
    except Exception as exc:
        error_message = (
            f"An error occurred while looking up data assigned to PID '{pid}'"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(f"{error_message}.") from exc

    if query_result_length == 0:
        raise DoesNotExist("PID is not attached to any data.")
    if query_result_length != 1:
        raise ApiError("PID must be unique.")

    return query_result[0]


@access_control(can_get_pid_for_data)
def get_pid_for_data(data_id, request):
    """Retrieve PID matching the document ID provided.

    Args:
        data_id:
        request: HttpRequest

    Returns:
    """
    try:
        # Retrieve the document passed as input and extra the PID field.
        data = data_api.get_by_id(data_id, request.user)

        # Return PID value from the document and the pid_path retrieved from
        # `PidSettings`
        pid_path_object = pid_path_api.get_by_template(
            data.template, request.user
        )
        pid_path = pid_path_object.path

        # If the pid_path does not exist in the document, exit early and return None
        if not is_dot_notation_in_dictionary(
            data.get_dict_content(), pid_path
        ):
            return None

        pid_value = get_value_from_dot_notation(
            data.get_dict_content(), pid_path
        )

        # If the field has an invalid PID, raise an exception.
        if not is_valid_pid_value(
            pid_value, settings.ID_PROVIDER_SYSTEM_NAME, settings.PID_FORMAT
        ):
            raise ApiError(f"Invalid PID in data '{data_id}'")

        return pid_value
    except Exception as exc:
        error_message = (
            f"An error occurred while looking up PID assigned to data "
            f"'{data_id}'"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise ApiError(f"{error_message}.") from exc



""" Unit tests for core_linked_records_app.utils.providers.local
"""

import json
from unittest import TestCase
from unittest.mock import patch

from rest_framework import status

from core_linked_records_app.utils.providers.local import LocalIdProvider
from core_main_app.commons import exceptions
from tests.mocks import MockResponse, MockLocalId


class TestLocalIdProviderIsIdAlreadyUsed(TestCase):
    """Test Local Id Provider Is Id Already Used"""

    def setUp(self) -> None:
        self.provider = LocalIdProvider("mock_provider")

    @patch("core_linked_records_app.utils.providers.local.LocalIdProvider.get")
    def test_calls_get_method(self, mock_get):
        """test_calls_get_method"""
        mock_get_value = MockResponse()
        mock_get_value.content = json.dumps({"message": "mock_message"})
        mock_get.return_value = mock_get_value

        self.provider.is_id_already_used("mock_record")
        mock_get.assert_called()

    @patch("core_linked_records_app.utils.providers.local.LocalIdProvider.get")
    def test_returns_true_if_successful(self, mock_get):
        """test_returns_true_if_successful"""
        mock_get_value = MockResponse()
        mock_get_value.content = json.dumps(
            {"message": self.provider.messages["success"]}
        )
        mock_get.return_value = mock_get_value

        self.assertTrue(self.provider.is_id_already_used("mock_record"))

    @patch("core_linked_records_app.utils.providers.local.LocalIdProvider.get")
    def test_returns_false_if_failure(self, mock_get):
        """test_returns_false_if_failure"""
        mock_get_value = MockResponse()
        mock_get_value.content = json.dumps(
            {"message": self.provider.messages["not_found"]}
        )
        mock_get.return_value = mock_get_value

        self.assertFalse(self.provider.is_id_already_used("mock_record"))


class TestLocalIdProviderGet(TestCase):
    """Test Local Id Provider Get"""

    def setUp(self) -> None:
        self.provider = LocalIdProvider("mock_provider")
        self.record = "mock_record"

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_non_existing_record_returns_404(self, mock_get_by_name):
        """test_non_existing_record_returns_404"""
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            "mock_does_not_exists"
        )
        self.assertEqual(
            self.provider.get(self.record).status_code,
            status.HTTP_404_NOT_FOUND,
        )

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_non_existing_record_returns_correct_content(
        self, mock_get_by_name
    ):
        """test_non_existing_record_returns_correct_content"""
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            "mock_does_not_exists"
        )
        self.assertEqual(
            json.loads(self.provider.get(self.record).content),
            {
                "message": self.provider.messages["not_found"],
                "record": self.record,
                "url": f"{self.provider.provider_lookup_url}/{self.record}",
            },
        )

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_existing_record_returns_200(self, mock_get_by_name):
        """test_existing_record_returns_200"""
        mock_get_by_name.return_value = None
        self.assertEqual(
            self.provider.get(self.record).status_code, status.HTTP_200_OK
        )

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_existing_record_returns_correct_content(self, mock_get_by_name):
        """test_existing_record_returns_correct_content"""
        mock_get_by_name.return_value = None
        self.assertEqual(
            json.loads(self.provider.get(self.record).content),
            {
                "message": self.provider.messages["success"],
                "record": self.record,
                "url": f"{self.provider.provider_lookup_url}/{self.record}",
            },
        )


class TestLocalIdProviderCreate(TestCase):
    """Test Local Id Provider Create"""

    def setUp(self) -> None:
        self.provider = LocalIdProvider("mock_provider")
        self.prefix = "mock_prefix"
        self.record = "mock_record"

    @patch("core_linked_records_app.system.local_id.api.insert")
    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider.is_id_already_used"
    )
    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider._generate_id"
    )
    def test_record_none_generate_new_record(
        self, mock_generate_id, mock_is_id_already_used, mock_localid_insert
    ):
        """test_record_none_generate_new_record"""
        mock_generate_id.return_value = self.record
        mock_is_id_already_used.return_value = False
        mock_localid_insert.return_value = None
        self.provider.create(self.prefix)

        self.assertTrue(mock_generate_id.called)
        self.assertTrue(mock_is_id_already_used.called)

    @patch("core_linked_records_app.system.local_id.api.insert")
    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider.is_id_already_used"
    )
    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider._generate_id"
    )
    def test_record_none_ensure_new_record_is_unique(
        self, mock_generate_id, mock_is_id_already_used, mock_localid_insert
    ):
        """test_record_none_checks_new_record_is_unique"""
        mock_generate_id.return_value = self.record
        mock_is_id_already_used.side_effect = [True, False]
        mock_localid_insert.return_value = None
        self.provider.create(self.prefix)

        self.assertEqual(mock_generate_id.call_count, 2)
        self.assertEqual(mock_is_id_already_used.call_count, 2)

    @patch("core_linked_records_app.system.local_id.api.insert")
    def test_non_unique_record_returns_409(self, mock_localid_insert):
        """test_non_unique_record_returns_409"""
        mock_localid_insert.side_effect = exceptions.NotUniqueError(
            "mock_not_unique_error"
        )
        response = self.provider.create(self.prefix, self.record)

        self.assertEqual(response.status_code, status.HTTP_409_CONFLICT)

    @patch("core_linked_records_app.system.local_id.api.insert")
    def test_non_unique_record_returns_correct_content(
        self, mock_localid_insert
    ):
        """test_non_unique_record_returns_correct_content"""
        mock_localid_insert.side_effect = exceptions.NotUniqueError(
            "mock_not_unique_error"
        )
        response = self.provider.create(self.prefix, self.record)

        self.assertEqual(
            json.loads(response.content),
            {
                "message": self.provider.messages["already_exist"],
                "record": f"{self.prefix}/{self.record}",
                "url": f"{self.provider.provider_lookup_url}/{self.prefix}/{self.record}",
            },
        )

    @patch("core_linked_records_app.system.local_id.api.insert")
    def test_successful_creation_returns_201(self, mock_localid_insert):
        """test_successful_creation_returns_201"""
        mock_localid_insert.return_value = None
        response = self.provider.create(self.prefix, self.record)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch("core_linked_records_app.system.local_id.api.insert")
    def test_successful_creation_returns_correct_content(
        self, mock_localid_insert
    ):
        """test_successful_creation_returns_correct_content"""
        mock_localid_insert.return_value = None
        response = self.provider.create(self.prefix, self.record)

        self.assertEqual(
            json.loads(response.content),
            {
                "message": self.provider.messages["success"],
                "record": f"{self.prefix}/{self.record}",
                "url": f"{self.provider.provider_lookup_url}/{self.prefix}/{self.record}",
            },
        )


class TestLocalIdProviderUpdate(TestCase):
    """Test Local Id Provider Update"""

    def setUp(self) -> None:
        self.provider = LocalIdProvider("mock_provider")
        self.record = "mock_record"

    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider.create"
    )
    def test_create_is_called(self, mock_create):
        """test_create_is_called"""
        mock_create.return_value = None
        self.provider.update(self.record)

        self.assertTrue(mock_create.called)

    @patch(
        "core_linked_records_app.utils.providers.local.LocalIdProvider.create"
    )
    def test_returns_correct_content(self, mock_create):
        """test_returns_correct_content"""
        mock_create.return_value = None
        response = self.provider.update(self.record)

        self.assertEqual(
            json.loads(response.content),
            {
                "record": self.record,
                "message": self.provider.messages["success"],
                "url": f"{self.provider.provider_lookup_url}/{self.record}",
            },
        )


class TestLocalIdProviderDelete(TestCase):
    """Test Local Id Provider Delete"""

    def setUp(self) -> None:
        self.provider = LocalIdProvider("mock_provider")
        self.record = "mock_record"

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_delete_is_called(self, mock_get_by_name):
        """test_successful_delete_returns_200"""
        mock_local_id = MockLocalId()
        mock_get_by_name.return_value = mock_local_id

        self.provider.delete(self.record)
        self.assertTrue(mock_local_id.delete.called)

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_successful_delete_returns_200(self, mock_get_by_name):
        """test_successful_delete_returns_200"""
        mock_local_id = MockLocalId()
        mock_get_by_name.return_value = mock_local_id

        response = self.provider.delete(self.record)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_successful_delete_returns_correct_content(self, mock_get_by_name):
        """test_successful_delete_returns_correct_content"""
        mock_local_id = MockLocalId()
        mock_get_by_name.return_value = mock_local_id

        response = self.provider.delete(self.record)
        self.assertEqual(
            json.loads(response.content),
            {
                "record": self.record,
                "message": self.provider.messages["success"],
                "url": f"{self.provider.provider_lookup_url}/{self.record}",
            },
        )

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_non_existing_record_returns_404(self, mock_get_by_name):
        """test_non_existing_record_returns_404"""
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            "mock_does_not_exist"
        )

        response = self.provider.delete(self.record)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch("core_linked_records_app.system.local_id.api.get_by_name")
    def test_non_existing_record_returns_correct_content(
        self, mock_get_by_name
    ):
        """test_non_existing_record_returns_correct_content"""
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            "mock_does_not_exist"
        )

        response = self.provider.delete(self.record)
        self.assertEqual(
            json.loads(response.content),
            {
                "record": self.record,
                "message": self.provider.messages["not_found"],
                "url": f"{self.provider.provider_lookup_url}/{self.record}",
            },
        )



""" Unit tests for `core_linked_records.components.oai_data.access_control`.
"""

from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_linked_records_app.components.oai_record import (
    access_control as oai_record_acl,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCanGetPidForData(TestCase):
    """Unit tests for `can_get_pid_for_data` function."""

    def setUp(self) -> None:
        mock_request = MagicMock()
        mock_request.user = create_mock_user("1")

        self.mock_kwargs = {
            "func": MagicMock(),
            "oai_record_id": "mock_oai_record_id",
            "request": mock_request,
        }

    @patch.object(oai_record_acl, "check_anonymous_access")
    def test_check_anonymous_access_called(self, mock_check_anonymous_access):
        """test_check_anonymous_access_called"""
        oai_record_acl.can_get_pid_for_data(**self.mock_kwargs)
        mock_check_anonymous_access.assert_called_with(
            self.mock_kwargs["request"].user
        )

    @patch.object(oai_record_acl, "check_anonymous_access")
    def test_check_anonymous_access_failure_raise_acl_error(
        self, mock_check_anonymous_access
    ):
        """test_check_anonymous_access_failure_raise_acl_error"""
        mock_check_anonymous_access.side_effect = AccessControlError(
            "mock_check_anonymous_access_mock_acl_error"
        )

        with self.assertRaises(AccessControlError):
            oai_record_acl.can_get_pid_for_data(**self.mock_kwargs)

    @patch.object(oai_record_acl, "check_anonymous_access")
    def test_successful_execution_returns_func_with_args(
        self,
        mock_check_anonymous_access,  # noqa, pylint: disable=unused-argument
    ):
        """test_successful_execution_returns_func_with_args"""
        self.assertEqual(
            oai_record_acl.can_get_pid_for_data(**self.mock_kwargs),
            self.mock_kwargs["func"](
                self.mock_kwargs["oai_record_id"], self.mock_kwargs["request"]
            ),
        )



""" Serializer classes for LocalID object
"""

from rest_framework.serializers import ModelSerializer

from core_linked_records_app.components.local_id.models import LocalId


class LocalIdSerializer(ModelSerializer):
    """LocalId serializer"""

    class Meta:
        """Meta"""

        model = LocalId
        fields = "__all__"
        read_only_fields = [
            "record_name",
            "record_object_class",
            "record_object_id",
        ]



""" Unit tests for core_linked_records_app.components.pid_settings.api.
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_linked_records_app.components.pid_settings import (
    api as pid_settings_api,
    access_control as pid_settings_acl,
)
from core_linked_records_app.system.pid_settings import (
    api as pid_settings_system_api,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestUpsert(TestCase):
    """Unit tests for `upsert` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_kwargs = {
            "pid_settings_object": MagicMock(),
            "user": create_mock_user("1", is_superuser=True),
        }

    @patch.object(pid_settings_system_api, "upsert")
    def test_pid_settings_system_api_upsert_called(self, mock_system_upsert):
        """test_pid_settings_system_api_upsert_called"""

        pid_settings_api.upsert(**self.mock_kwargs)
        mock_system_upsert.assert_called_with(
            self.mock_kwargs["pid_settings_object"]
        )

    @patch.object(pid_settings_system_api, "upsert")
    def test_returns_pid_settings_system_api_upsert_value(
        self, mock_system_upsert
    ):
        """test_returns_pid_settings_system_api_upsert_value"""
        expected_result = "mock_upsert"
        mock_system_upsert.return_value = expected_result

        self.assertEqual(
            pid_settings_api.upsert(**self.mock_kwargs), expected_result
        )


class TestGet(TestCase):
    """Unit tests for `get` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_user = create_mock_user("1")

    @patch.object(pid_settings_system_api, "get")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_pid_settings_system_api_get_called(
        self, mock_check_has_perm, mock_system_get
    ):
        """test_pid_settings_system_api_get_called"""
        mock_check_has_perm.return_value = True

        pid_settings_api.get(self.mock_user)
        mock_system_get.assert_called()

    @patch.object(pid_settings_system_api, "get")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_returns_pid_settings_system_api_get_value(
        self, mock_check_has_perm, mock_system_get
    ):
        """test_returns_pid_settings_system_api_get_value"""
        mock_check_has_perm.return_value = True
        expected_result = "mock_get"
        mock_system_get.return_value = expected_result

        self.assertEqual(pid_settings_api.get(self.mock_user), expected_result)



""" Unit tests for `core_linked_records.rest.pid_settings.serializers`.
"""

from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from core_linked_records_app.rest.pid_settings import (
    serializers as pid_settings_serializers,
)


class TestPidSettingsSerializerUpdate(TestCase):
    """Unit tests for `PidSettingsSerializer.update` method."""

    def setUp(self) -> None:
        """setUp"""
        self.context = {"request": MagicMock()}
        self.mock_kwargs = {
            "instance": Mock(),
            "validated_data": {"auto_set_pid": True},
        }

    @patch.object(pid_settings_serializers, "pid_settings_api")
    def test_pid_settings_system_api_upsert_called(
        self, mock_pid_settings_api
    ):
        """test_pid_settings_system_api_upsert_called"""
        serializer = pid_settings_serializers.PidSettingsSerializer()
        serializer._context = self.context
        serializer.update(**self.mock_kwargs)

        mock_pid_settings_api.upsert.assert_called_with(
            self.mock_kwargs["instance"], self.context["request"].user
        )

    @patch.object(pid_settings_serializers, "pid_settings_api")
    def test_succesful_execution_returns_pid_settings_system_api_upsert(
        self, mock_pid_settings_api
    ):
        """test_pid_settings_system_api_upsert_called"""
        expected_value = Mock()
        mock_pid_settings_api.upsert.return_value = expected_value
        serializer = pid_settings_serializers.PidSettingsSerializer()
        serializer._context = self.context

        self.assertEqual(serializer.update(**self.mock_kwargs), expected_value)



""" PID utilities related to data objects.
"""

import json
import logging

from core_linked_records_app.utils import exceptions
from core_linked_records_app.utils import (
    xml as pid_xml_utils,
    json as pid_json_utils,
    dict as pid_dict_utils,
)
from core_main_app.components.template.models import Template
from core_main_app.utils.json_utils import load_json_string
from xml_utils.commons.exceptions import XPathError
from xml_utils.xpath import create_tree_from_xpath
from xml_utils.xsd_tree.xsd_tree import XSDTree

logger = logging.getLogger(__name__)


def set_pid_value_for_data(data, pid_path, pid_value):
    """Set the document PID into XML data and update `content` in place.

    Args:
        data:
        pid_path:
        pid_value:
    """
    if data.template.format == Template.XSD:
        pid_xpath = pid_xml_utils.get_xpath_with_target_namespace(
            pid_xml_utils.get_xpath_from_dot_notation(pid_path),
            data.template.content,
        )

        target_namespace = pid_xml_utils.get_target_namespace_for_xsd_string(
            data.template.content
        )
        xml_tree = XSDTree.build_tree(data.content)

        xml_tree = create_tree_from_xpath(
            pid_xpath, xml_tree, target_namespace
        )
        pid_xml_utils.set_value_at_xpath(
            xml_tree, pid_xpath, pid_value, target_namespace
        )
        data.content = XSDTree.tostring(xml_tree)
    elif data.template.format == Template.JSON:
        json_content = load_json_string(data.content)
        json_content = pid_json_utils.set_value_at_dict_path(
            json_content, pid_path, pid_value
        )
        data.content = json.dumps(json_content)
    else:
        error_message = "Cannot create PID. Invalid template format."
        logger.error(error_message)
        raise exceptions.PidCreateError(error_message)

    # Update the whole document with the updated XML content
    data.convert_to_file()
    data.convert_to_dict()


def get_pid_value_for_data(data, pid_path):
    """Retrieve value located at `pid_path` of the data passed in parameter.

    Args:
        data:
        pid_path:

    Returns:
        str - Persitstent identifier
    """
    if data.template.format == Template.XSD:
        # Transform the dot notation path into an XML XPath.
        pid_path = pid_xml_utils.get_xpath_with_target_namespace(
            pid_xml_utils.get_xpath_from_dot_notation(pid_path),
            data.template.content,
        )

        target_namespace = pid_xml_utils.get_target_namespace_for_xsd_string(
            data.template.content
        )
        xml_tree = XSDTree.build_tree(data.content)

        try:  # Get the PID from the `pid_path` value
            pid_value = pid_xml_utils.get_value_at_xpath(
                xml_tree, pid_path, target_namespace
            )
        except XPathError as xpath_error_exc:  # PID path not found in document
            if not pid_xml_utils.can_create_value_at_xpath(
                data.content,
                data.template.content,
                pid_path,
                "http://sample_pid.org",
            ):
                raise exceptions.PidCreateError(
                    f"Cannot create pid value at {pid_path}"
                ) from xpath_error_exc
            pid_value = None
    elif data.template.format == Template.JSON:
        json_content = load_json_string(data.content)
        pid_value = pid_dict_utils.get_value_from_dot_notation(
            json_content, pid_path
        )

        # PID path has not been found in document and cannot be created.
        if (
            pid_value is None
            and not pid_json_utils.can_create_value_at_dict_path(
                json_content,
                data.template.content,
                pid_path,
                "http://sample_pid.org",
            )
        ):
            raise exceptions.PidCreateError(
                f"Cannot create pid value at {pid_path}"
            )
    else:
        error_message = "Cannot create PID. Invalid template format."
        logger.error(error_message)
        raise exceptions.InvalidPidError(error_message)

    # Return the PID and clean it up if it ends with a '/'.
    return (
        pid_value[:-1]
        if isinstance(pid_value, str) and pid_value.endswith("/")
        else pid_value
    )



""" Permission tests for core_linked_records_app.rest.providers.views
"""

import json
from unittest import TestCase
from unittest.mock import patch

from rest_framework import status

from core_linked_records_app import settings
from core_linked_records_app.rest.providers import views as providers_views
from core_linked_records_app.utils.providers import ProviderManager
from core_main_app.rest.data.serializers import DataSerializer
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests import mocks


class TestProviderRecordViewPost(TestCase):
    """Test Provider Record View Post"""

    @staticmethod
    def _send_request(mock_provider_manager_get, mock_user):
        mock_provider_manager_get.return_value = mocks.MockProviderManager(
            create_result=mocks.MockResponse(
                status_code=status.HTTP_201_CREATED
            )
        )
        return RequestMock.do_request_post(
            providers_views.ProviderRecordView.as_view(),
            mock_user,
            param={
                "provider": "local",
                "record": f"{settings.ID_PROVIDER_PREFIXES[0]}/mock_record",
            },
        )

    @patch.object(ProviderManager, "get")
    def test_anonymous_returns_201(self, mock_provider_manager_get):
        """test_anonymous_returns_201"""

        response = self._send_request(mock_provider_manager_get, None)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(ProviderManager, "get")
    def test_authenticated_returns_201(self, mock_provider_manager_get):
        """test_authenticated_returns_201"""

        mock_user = create_mock_user("1")

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(ProviderManager, "get")
    def test_staff_returns_201(self, mock_provider_manager_get):
        """test_staff_returns_201"""

        mock_user = create_mock_user("1", is_staff=True)

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestProviderRecordViewPut(TestCase):
    """Test Provider Record View Put"""

    @staticmethod
    def _send_request(mock_provider_manager_get, mock_user):

        mock_provider_manager_get.return_value = mocks.MockProviderManager(
            update_result=mocks.MockResponse()
        )
        return RequestMock.do_request_put(
            providers_views.ProviderRecordView.as_view(),
            mock_user,
            param={
                "provider": "local",
                "record": f"{settings.ID_PROVIDER_PREFIXES[0]}/mock_record",
            },
        )

    @patch.object(ProviderManager, "get")
    def test_anonymous_returns_200(self, mock_provider_manager_get):
        """test_anonymous_returns_200"""

        response = self._send_request(mock_provider_manager_get, None)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(ProviderManager, "get")
    def test_authenticated_returns_200(self, mock_provider_manager_get):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(ProviderManager, "get")
    def test_staff_returns_200(self, mock_provider_manager_get):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestProviderRecordViewGet(TestCase):
    """Test Provider Record View Get"""

    @staticmethod
    def _send_request(
        mock_provider_manager_get,
        mock_get_data_by_pid,
        mock_data_serializer,
        mock_user,
    ):
        mock_provider_manager_get.return_value = mocks.MockProviderManager(
            get_result=mocks.MockResponse(
                content=json.dumps({"url": "mock_url"})
            )
        )
        mock_get_data_by_pid.return_value = None
        mock_data_serializer.return_value = mocks.MockSerializer()
        return RequestMock.do_request_get(
            providers_views.ProviderRecordView.as_view(),
            mock_user,
            param={
                "provider": "local",
                "record": f"{settings.ID_PROVIDER_PREFIXES[0]}/mock_record",
            },
        )

    @patch.object(DataSerializer, "__new__")
    @patch.object(providers_views, "get_data_by_pid")
    @patch.object(ProviderManager, "get")
    def test_anonymous_returns_200(
        self,
        mock_provider_manager_get,
        mock_get_data_by_pid,
        mock_data_serializer,
    ):
        """test_anonymous_returns_200"""

        response = self._send_request(
            mock_provider_manager_get,
            mock_get_data_by_pid,
            mock_data_serializer,
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "__new__")
    @patch.object(providers_views, "get_data_by_pid")
    @patch.object(ProviderManager, "get")
    def test_authenticated_returns_200(
        self,
        mock_provider_manager_get,
        mock_get_data_by_pid,
        mock_data_serializer,
    ):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        response = self._send_request(
            mock_provider_manager_get,
            mock_get_data_by_pid,
            mock_data_serializer,
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "__new__")
    @patch.object(providers_views, "get_data_by_pid")
    @patch.object(ProviderManager, "get")
    def test_staff_returns_200(
        self,
        mock_provider_manager_get,
        mock_get_data_by_pid,
        mock_data_serializer,
    ):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        response = self._send_request(
            mock_provider_manager_get,
            mock_get_data_by_pid,
            mock_data_serializer,
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestProviderRecordViewDelete(TestCase):
    """Test Provider Record View Delete"""

    @staticmethod
    def _send_request(mock_provider_manager_get, mock_user):
        mock_provider_manager_get.return_value = mocks.MockProviderManager(
            delete_result=mocks.MockResponse(
                status_code=status.HTTP_204_NO_CONTENT
            )
        )
        return RequestMock.do_request_delete(
            providers_views.ProviderRecordView.as_view(),
            mock_user,
            param={
                "provider": "local",
                "record": f"{settings.ID_PROVIDER_PREFIXES[0]}/mock_record",
            },
        )

    @patch.object(ProviderManager, "get")
    def test_anonymous_returns_204(self, mock_provider_manager_get):
        """test_anonymous_returns_204"""

        response = self._send_request(mock_provider_manager_get, None)

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(ProviderManager, "get")
    def test_authenticated_returns_204(self, mock_provider_manager_get):
        """test_authenticated_returns_204"""

        mock_user = create_mock_user("1")

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(ProviderManager, "get")
    def test_staff_returns_204(self, mock_provider_manager_get):
        """test_staff_returns_204"""

        mock_user = create_mock_user("1", is_staff=True)

        response = self._send_request(mock_provider_manager_get, mock_user)

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



""" Serializers for calls related to `PidPath` model.
"""

from rest_framework import serializers

from core_linked_records_app.components.pid_path.models import PidPath


class PidPathSerializer(serializers.ModelSerializer):
    """Serializer for the `PidPath` model."""

    class Meta:
        """Meta"""

        model = PidPath
        fields = "__all__"
